define([
	"require",			// require.toUrl
	"dojo/_base/kernel", // dojo.config
	".",
	"dojo/_base/connect", // dojo.connect dojo.disconnect
	"dojo/_base/html", // dojo.create dojo.style
	"dojo/_base/lang", // dojo.extend
	"dojo/_base/sniff", // dojo.isIE dojo.isMoz dojo.isQuirks
	"dojo/_base/window" // dojo.doc.createElement
], function(require, dojo, dijit){

	// module:
	//		dijit/BackgroundIFrame
	// summary:
	//		new dijit.BackgroundIframe(node)
	//		Makes a background iframe as a child of node, that fills
	//		area (and position) of node

	// TODO: remove _frames, it isn't being used much, since popups never release their
	// iframes (see [22236])
	var _frames = new function(){
		// summary:
		//		cache of iframes

		var queue = [];

		this.pop = function(){
			var iframe;
			if(queue.length){
				iframe = queue.pop();
				iframe.style.display="";
			}else{
				if(dojo.isIE < 9){
					var burl = dojo.config["dojoBlankHtmlUrl"] || require.toUrl("dojo/resources/blank.html") || "javascript:\"\"";
					var html="<iframe src='" + burl + "' role='presentation'"
						+ " style='position: absolute; left: 0px; top: 0px;"
						+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
					iframe = dojo.doc.createElement(html);
				}else{
					iframe = dojo.create("iframe");
					iframe.src = 'javascript:""';
					iframe.className = "dijitBackgroundIframe";
					iframe.setAttribute("role", "presentation");
					dojo.style(iframe, "opacity", 0.1);
				}
				iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			}
			return iframe;
		};

		this.push = function(iframe){
			iframe.style.display="none";
			queue.push(iframe);
		}
	}();


	dijit.BackgroundIframe = function(/*DomNode*/ node){
		// summary:
		//		For IE/FF z-index schenanigans. id attribute is required.
		//
		// description:
		//		new dijit.BackgroundIframe(node)
		//			Makes a background iframe as a child of node, that fills
		//			area (and position) of node

		if(!node.id){ throw new Error("no id"); }
		if(dojo.isIE || dojo.isMoz){
			var iframe = (this.iframe = _frames.pop());
			node.appendChild(iframe);
			if(dojo.isIE<7 || dojo.isQuirks){
				this.resize(node);
				this._conn = dojo.connect(node, 'onresize', this, function(){
					this.resize(node);
				});
			}else{
				dojo.style(iframe, {
					width: '100%',
					height: '100%'
				});
			}
		}
	};

	dojo.extend(dijit.BackgroundIframe, {
		resize: function(node){
			// summary:
			// 		Resize the iframe so it's the same size as node.
			//		Needed on IE6 and IE/quirks because height:100% doesn't work right.
			if(this.iframe){
				dojo.style(this.iframe, {
					width: node.offsetWidth + 'px',
					height: node.offsetHeight + 'px'
				});
			}
		},
		destroy: function(){
			// summary:
			//		destroy the iframe
			if(this._conn){
				dojo.disconnect(this._conn);
				this._conn = null;
			}
			if(this.iframe){
				_frames.push(this.iframe);
				delete this.iframe;
			}
		}
	});

	return dijit.BackgroundIframe;
});